require 'time'
require 'openSSL'
require 'base64'
require 'digest'
require 'json'
require 'net/http'
require 'uri'

class Paytrail
    def initialize(apiName, apiKey, secret)
        @apiName = apiName
        @apiKey = apiKey
        @secret = secret
    end

    def makeTimestamp()
        Time.now.iso8601
    end

    def makeAuthorization(method, location, timestamp, contentMD5)
        signatureData = "%s\n%s\n%s %s\n%s\n%s" % [method, location, @apiName, @apiKey, timestamp, contentMD5]
        digest = OpenSSL::Digest::SHA256.new
        hmac = OpenSSL::HMAC.digest(digest, @secret, signatureData)

        Base64.strict_encode64(hmac)
    end

    def makeContentMD5(content)
        Digest::MD5.base64digest content
    end

    def makeHeaders(method, location, timestamp, body)
        {
            'Timestamp' => timestamp,
            'Content-MD5' => makeContentMD5(body),
            'Authorization' => "%s %s:%s" % [@apiName, @apiKey, makeAuthorization(method, location, timestamp, makeContentMD5(body))]
        }
    end

    def post(uri, headers, body)
        uri = URI(uri)
        request = Net::HTTP::Post.new(uri, headers)
        request.body = body

        makeRequest(uri, request)
    end

    def delete(uri, headers)
        uri = URI(uri)
        request = Net::HTTP::Delete.new(uri, headers)

        makeRequest(uri, request)
    end

    def makeRequest(uri, request)
        http = Net::HTTP.start(uri.hostname, uri.port)

        http.request(request)
    end
end